/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * Copyright (c) 2004 Nokia. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Nokia nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef Cookie_h
#define Cookie_h

#include <glib.h>

#include "own_gcharptr.h"
#include "gnet/uri.h"

class CookieParser;

class Cookie
{
    friend class CookieParser;
public:
    Cookie(const gchar* name, const gchar* value, bool);
    ~Cookie();

    const gchar* headerString();

    const gchar* name() const { return m_name; }
    const gchar* value() const { return m_value; }
    const gchar* comment() const { return m_comment; }
    const gchar* commentURL() const { return m_commentURL; }
    const gchar* domain() const { return m_domain; }
    const gchar* path() const { return m_path; }
    guint maxAge() const { return m_maxAge; }
    gint version() const { return m_version; }
    bool secure() const { return m_secure; }
    bool discard() const { return m_discard; }
    const int* ports() const { return m_ports; }
    int nports() const { return m_nports;}

    bool isValidForURL(GURI*);
    bool shouldDelete();
    bool isExpired();

    const gchar* realCookieString() const { return m_setCookieString; }
private:
    void parseCookie(gchar* cookie);
    void setInvalid();

private:
    own_gcharptr m_setCookieString;
    own_gcharptr m_headerString;
    own_gcharptr m_name;
    own_gcharptr m_value;
    own_gcharptr m_comment;
    own_gcharptr m_commentURL;
    own_gcharptr m_domain;
    own_gcharptr m_path;

    own_gcharptr m_realPath;
    int m_version;
    int m_maxAge;
    bool m_secure;
    bool m_discard;

    GTimeVal m_received;
    int *m_ports;
    int m_nports;
    bool m_forceInvalid;

};

#endif
