/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* Copyright (C) 2007 Kimmo Kinnunen<kimmok@iki.fi>
 * bsd license
 */
#ifndef CookieParser_h
#define CookieParser_h

#include <glib.h>
class Cookie;


class CookieParser
{
public:
	CookieParser();
	~CookieParser();
    /** steal the cookie and set the cell to 0 for ownership change*/
	Cookie** parse(const gchar* cookies);

private:
	enum State { 
		// state tokens
		COMMENT, COMMENT_URL, DISCARD, DOMAIN, MAXAGE, PATH, PORT, SECURE, VERSION, UNKNOWN,

		// other states
		END, ERROR, COOKIE, MAYBE_COOKIE, MAYBE_COOKIE_AV, COOKIE_AV, PORT_LIST, PORT_LIST_QUOTE
	};

    static const int s_cookiesSz = 10;
	static const int s_portsSz  = 10;

	void readNext(gchar);
	bool ifNext(gchar);
	gchar* readToken(bool isPath = false);
	gchar* readValue(bool isPath = false);
	int readInt();
	int readPlainInt();
	gchar* readQuote();
	State readStateToken();
	void readUnknown();

	void emitNameValue(gchar* name, gchar* value);
	void emitPortListStart();
	void emitPortListEnd(bool error);
	void emitPort(int port);
	void emitVersion(int version);
	void emitMaxAge(int age);
	void emitComment(gchar* comment);
	void emitCommentURL(gchar* url);
	void emitPath(gchar* path);
	void emitDomain(gchar* domain);
	void emitSecure();
	void emitDiscard();
	void emitUnknown();

	void deleteCookies();

    int m_ncookies;
	Cookie* m_cookies[s_cookiesSz];
	Cookie* m_curCookie;

	int m_ports[s_portsSz];
	int m_nports;

	bool m_err;
	gchar* m_value;
};
#endif
