/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * Copyright (c) 2007 Hiroyuki Ikezoe All rights reserved.
 * Copyright (c) 2007 Kouhei Sutou All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of authors nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef PangoTextRenderer_h
#define PangoTextRenderer_h

#include <pango/pango.h>

#include <NRCore/WebCoreTextRenderer.h>

class PangoNSFont;
class PangoTextRendererFactory;

// class assumes that in each point, CGContext's clip region is relative to origin
// and sets the gc clip origin to 0,0 after clip origin changes
class PangoTextRenderer : public WebCoreTextRenderer
{
    friend class PangoTextRendererFactory;

public:

    float floatWidthForRun(const WebCoreTextRun *run, const WebCoreTextStyle *style, float *widths);

    void setContext(CGContextRef context);

    void drawRun(const WebCoreTextRun *run, const WebCoreTextStyle *style, int x, int y);
    void drawHighlightForRun(const WebCoreTextRun *run, const WebCoreTextStyle *style, const WebCoreTextGeometry *geom);
    void drawLineForCharacters(int x, int y, float yOffset, int width, GdkColor* color);

    int pointToOffset(const WebCoreTextRun *run, const WebCoreTextStyle *style, int x, bool reversed, bool includePartialGlyphs);

    void detach() {m_owner = 0;}

    ~PangoTextRenderer();

protected:
    PangoTextRenderer(PangoTextRendererFactory* owner, PangoNSFont *font);

private:
    // prevent copying
    PangoTextRenderer(const PangoTextRenderer&);
    PangoTextRenderer &operator=(const PangoTextRenderer&);

    void setPangoAttributes(const WebCoreTextStyle *style, PangoLayout *layout);
    PangoLayout *getPangoLayoutForGdkDrawable(GdkDrawable *drawable);
    PangoLayout *getPangoLayout();
    gchar *convertUniCharToUTF8(const UniChar *characters, int from, int to);

private:
    PangoTextRendererFactory* m_owner;
    PangoNSFont *m_font;
    CGContextRef m_context;
    gint m_xoffset;
	gint m_yoffset;
};

#endif
