/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * Copyright (c) 2007 Hiroyuki Ikezoe All rights reserved.
 * Copyright (c) 2007 Kouhei Sutou All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of authors nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef PangoTextRendererFactory_h
#define PangoTextRendererFactory_h

#include <glib.h>
#include <pango/pango.h>

#include <NRCore/WebCoreTextRendererFactory.h>
#include "PangoTextRenderer.h"

class PangoTextRendererFactory;


class PangoNSFont : public NSFont
{
public:
	PangoNSFont(PangoTextRendererFactory* creator, PangoFontDescription *desc, guint hash);
	~PangoNSFont();

	void detach() { creator = 0; }
	int ascent;
	int descent;
	int lineSpacing;
	float xHeight;
	int spaceWidth;
	bool fixedPitch;
	guint hash;
	PangoFontDescription *desc;
	PangoTextRendererFactory* creator;
};

class PangoTextRendererFactory : public WebCoreTextRendererFactory
{
    friend class WebCoreTextRendererFactory; // WebCoreTextRendererFactory::sharedFactory

public:
	NSFont* fontWithFamilies(const gchar * const * families, NSFontTraitMask traits, float size);

	PangoTextRenderer* rendererWithFont(NSFont* font, bool isPrinterFont);     // co-variant return type

	bool isFontFixedPitch(NSFont* f);

	void rendererDeleted(PangoTextRenderer* renderer);
	void fontDeleted(PangoNSFont* font);

protected:
	PangoTextRendererFactory();
	~PangoTextRendererFactory();

private:
	PangoTextRendererFactory(const PangoTextRendererFactory&);
	PangoTextRendererFactory& operator=(const PangoTextRendererFactory&);

private:
	GHashTable* m_nsfonts;
};

#endif
