/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: f; c-basic-offset: 4 -*- */
/*
 * Copyright (c) 2004 Nokia. All rights reserved.
 * Copyright (c) 2007 Kimmo Kinnunen. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Nokia nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WebiEventListener_h
#define WebiEventListener_h

#include "osb.h"
#include "own_gcharptr.h"

class WebiRoot;
class RootImpl;

class WebiEventListener : public OSB::FrameLoadDelegate, public OSB::UIDelegate, public OSB::ResourceLoadDelegate
{
public:
	WebiEventListener (WebiRoot* root);

	~WebiEventListener();

	/** FrameLoadDelegate*/
	void onClientRedirectReceived(FrameImpl* target, const gchar* to);
	void onServerRedirected(FrameImpl* target, const gchar* to);

	void onFrameLoadStarted(FrameImpl* target);
	void onFrameLoadFinished(FrameImpl* target, int status);

	void onCommitLoad(FrameImpl* target);
	bool onSetCookie(FrameImpl* target, const Cookie * cookie, const gchar* url);

	/** ResourceLoadDelegate*/
	void onResourceLoadStarted(FrameImpl* target, const OSB::ResourceStatus *status);
	void onResourceLoadHeaders(FrameImpl* target, const OSB::ResourceStatus *status);
	void onResourceLoadStatus(FrameImpl* target, const OSB::ResourceStatus *status);
	void onResourceLoadFinished(FrameImpl* target, const OSB::ResourceStatus *status);

	void onTitleChanged(FrameImpl* target, const gchar* title);

	/** UIDelegate */
	void setWindowProperties(FrameImpl* sender, OSB::Features &features);
	void getWindowSize(FrameImpl* frame, GtkAllocation * size, bool isContentSize);
	void setWindowSize(FrameImpl* frame, GtkRequisition * size, bool isContentSize);
	void onSelectionChanged(FrameImpl* target);

	bool authPanel(FrameImpl* target, const gchar* url, const gchar *realm, gchar **user, gchar **password);


	/* own  methods*/
	const gchar* statusText() { return status; }
	const gchar* location() { return loc; }
	const gchar* title() { return pageTitle; }
	void clearState();

private:
	WebiRoot* root;
	gchar* status;
	gchar* loc;
	gchar* pageTitle;
	int strSz;

};

#endif
