/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: f; c-basic-offset: 4 -*- */
/*
 * Copyright (c) 2004 Nokia. All rights reserved.
 * Copyright (c) 2007 Kimmo Kinnunen<kimmok@ikif.i>. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Nokia nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "deprecated-signal-sender.h"
#include "webkit-gtk/webkit-ui-delegate.h"
#include "webkit-gtk/webkit-url-request.h"
#include "webkit-gtk/webkit-element-info.h"
#include "webi.h"

typedef struct DeprecatedSignalSender_ DeprecatedSignalSender;
typedef struct DeprecatedSignalSenderClass_ DeprecatedSignalSenderClass;

struct DeprecatedSignalSenderClass_ {
	GObjectClass parent_class;
};

struct DeprecatedSignalSender_ {
	GObject parent;
	GObject* receiver;
	gchar* js_input_text;
	gchar* status_text;
};

#define TYPE_DEPRECATED_SIGNAL_SENDER (deprecated_signal_sender_get_type())


static void deprecated_signal_sender_class_init (DeprecatedSignalSenderClass *klass);
static void deprecated_signal_sender_ui_delegate_init(WebkitUIDelegateInterface *iface, gpointer user_data);
static void deprecated_signal_sender_finalize(GObject* self);

G_DEFINE_TYPE_WITH_CODE (DeprecatedSignalSender, deprecated_signal_sender, G_TYPE_OBJECT,
						 G_IMPLEMENT_INTERFACE (WEBKIT_TYPE_UI_DELEGATE,
												deprecated_signal_sender_ui_delegate_init))
#define GET_RSELF(self) DeprecatedSignalSender* rself = (DeprecatedSignalSender*) (self);

static void deprecated_signal_sender_class_init (DeprecatedSignalSenderClass *klass)
{
	/* bind finalizing function */
	G_OBJECT_CLASS (klass)->finalize = (GObjectFinalizeFunc) deprecated_signal_sender_finalize;
}


static void deprecated_signal_sender_init (DeprecatedSignalSender* self)
{
	self->js_input_text = 0;
	self->status_text = 0;
}

static void deprecated_signal_sender_finalize(GObject* self)
{
	GET_RSELF(self);

	if (rself->receiver)
		g_object_unref(rself->receiver);
}

WebkitUIDelegate* deprecated_signal_sender_new(GObject* receiver)
{
	DeprecatedSignalSender* self = (DeprecatedSignalSender*) g_object_new (TYPE_DEPRECATED_SIGNAL_SENDER, NULL);
	self->receiver = receiver;
	g_object_ref(self->receiver);

	return WEBKIT_UI_DELEGATE (self);
}


WebkitView* dss_create_view_with_request(WebkitUIDelegate* self, WebkitView* sender, WebkitURLRequest* request)
{
	/* this will send the signal to both GtkWebkitViewSignalStub and Webi
	 * Webi has different return type (ie Webi* instead of WebkitView*) but
	 * this is handled dynamically in frameimpl.cpp atm.
	 */
	GET_RSELF(self);
	WebkitView* new_engine = 0;
    g_signal_emit_by_name (rself->receiver, "req-new-window", webkit_url_request_get_url_str(request), &new_engine);
	return new_engine;
}

void dss_close(WebkitUIDelegate* self, WebkitView* sender)
{
	GET_RSELF(self);

    g_signal_emit_by_name(rself->receiver, "close-window");
}

void dss_show_view(WebkitUIDelegate* self, WebkitView* sender)
{
	GET_RSELF(self);

    g_signal_emit_by_name(rself->receiver, "show-window");
}

void dss_set_status_text(WebkitUIDelegate* self, WebkitView* sender, const gchar* status_text)
{
	GET_RSELF(self);

	if (rself->status_text)
		g_free(rself->status_text);

	rself->status_text = 0;

	if (status_text)
		rself->status_text = g_strdup(status_text);

    g_signal_emit_by_name(rself->receiver, "status_text");
}

const gchar* dss_get_status_text(WebkitUIDelegate* self, WebkitView* sender)
{
	GET_RSELF(self);

	return rself->status_text;
}

void dss_run_javascript_alert_panel_with_message(WebkitUIDelegate* self, WebkitView* sender, const gchar* message, WebkitFrame *initiator_frame)
{
    WebiPromptArgs args = {WEBI_ALERT,0,};
	GET_RSELF(self);
    args.msg = message;

    g_signal_emit_by_name (rself->receiver, "req-js-prompt", &args);
}

gboolean dss_run_javascript_confirm_panel_with_message(WebkitUIDelegate* self, WebkitView* sender, const gchar* message, WebkitFrame *initiator_frame)
{
    WebiPromptArgs args = {WEBI_CONFIRM,0,};
	GET_RSELF(self);
    args.msg = message;

    g_signal_emit_by_name (rself->receiver, "req-js-prompt", &args);

    return (args.out_ok_pressed == TRUE);

}

gchar* dss_run_javascript_input_panel_with_prompt(WebkitUIDelegate* self, WebkitView* sender, const gchar* message, const gchar* default_text, WebkitFrame *initiator_frame)
{
    WebiPromptArgs args = {WEBI_INPUT, 0,};
	GET_RSELF(self);

    args.msg = message;

    args.default_input = default_text;

    g_signal_emit_by_name (rself->receiver, "req-js-prompt", &args);

    /* clear possible old values */
	if (rself->js_input_text) {
		g_free (rself->js_input_text);
		rself->js_input_text = 0;
	}

    /* copy new results
     * WebiInputArgs::out_input must be allocated with g_new, and will be owned and released by
     * us */
    if (args.out_input)
        rself->js_input_text = args.out_input; /* no g_strdup() by api convention*/

    /* propagate results to caller. results will be valid until next call to this method */
    return rself->js_input_text;
}

gboolean dss_mouse_action_on_element(WebkitUIDelegate* self, WebkitView* sender, WebkitElementInfo* element_info, WebkitMouseEventInfo* mouse_info)
{
	GET_RSELF(self);
	if (element_info) {
		g_signal_emit_by_name(rself->receiver,
							  "mouse-over",
							  webkit_element_info_get_link_title(element_info),
							  webkit_element_info_get_link_label(element_info),
							  webkit_element_info_get_link_url(element_info),
							  webkit_element_info_get_link_target(element_info));
	}
	return FALSE;
}

static void deprecated_signal_sender_ui_delegate_init(WebkitUIDelegateInterface *iface, gpointer user_data)
{
	iface->create_view_with_request = dss_create_view_with_request;

	iface->show_view = dss_show_view;

	iface->close_view = dss_close;

	iface->set_status_text = dss_set_status_text;

	iface->get_status_text = dss_get_status_text;

	iface->run_javascript_alert_panel_with_message = dss_run_javascript_alert_panel_with_message;

	iface->run_javascript_confirm_panel_with_message = dss_run_javascript_confirm_panel_with_message;

	iface->run_javascript_input_panel_with_prompt = dss_run_javascript_input_panel_with_prompt;

	iface->mouse_action_on_element = dss_mouse_action_on_element;
}
