/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/* Copyright (c) 2007 Kimmo Kinnunen. All rights reserved.
 * bsd licence
 */
#include "webi-auth.h"
#include "string.h"
#include "webi-url.h"

struct WebiAuthList_
{
	GHashTable* auths;
};

static void webi_auth_list_entry_destroy(WebiAuthListEntry* entry);

static gchar*
make_key(const gchar* url, const gchar* realm)
{
	g_return_val_if_fail (url, NULL);
	g_return_val_if_fail (realm, NULL);
	gchar* result;
	gchar* dkey;
	gchar* host;
	host = webi_url_str_get_host(url);
	dkey = g_ascii_strdown(host, -1);
	g_free (host);
	result = g_strconcat(dkey, realm, NULL);
	g_free (dkey);

	return result;
}

WebiAuthList*
webi_auth_list_new()
{
	WebiAuthList* self;
	self = g_new0(WebiAuthList, 1);
	self->auths = g_hash_table_new_full(g_str_hash, g_str_equal, (GDestroyNotify) g_free, (GDestroyNotify) webi_auth_list_entry_destroy);
	return self;
}

void
webi_auth_list_destroy(WebiAuthList* self)
{
	g_return_if_fail(self);

	g_hash_table_destroy(self->auths);
	g_free(self);
}

void
webi_auth_list_insert(WebiAuthList* self, const gchar* url, const gchar* realm, WebiAuthListEntry* entry)
{
	g_return_if_fail(self);
	g_return_if_fail(url);
	g_return_if_fail(realm);
	g_return_if_fail(entry);

	gchar* key = make_key (url, realm);
	if (key)
		g_hash_table_insert (self->auths, key, entry);
	else
		webi_auth_list_entry_destroy(entry);
}


WebiAuthListEntry*
webi_auth_list_lookup(WebiAuthList* self, const gchar* url, const gchar* realm)
{
	WebiAuthListEntry* entry;

	g_return_val_if_fail(self, NULL);
	g_return_val_if_fail(url, NULL);
	g_return_val_if_fail(realm, NULL);

	gchar* key = make_key (url, realm);
	if (key) {
		entry = g_hash_table_lookup (self->auths, key);
		g_free (key);
		return entry;
	}

	return NULL;
}

WebiAuthListEntry*
webi_auth_list_entry_new()
{
	return g_new0(WebiAuthListEntry, 1);
}

static void
webi_auth_list_entry_destroy(WebiAuthListEntry* entry)
{
	g_free(entry);
}
