/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (c) 2004 Nokia. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Nokia nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <string.h> /* memset() */
#include <assert.h>

#include <glib/gprintf.h>

#include "webi.h"

#include "webi-private.h"
#include "webi-marshal.h"

#include "rootimpl.h"
#include "frameimpl.h"
#include "osb.h"
#include "BridgeImpl.h"
#include "WebiRoot.h"
#include "deprecated-signal-sender.h"


static void webi_class_init (WebiClass * c) G_GNUC_UNUSED;
static void webi_init (Webi * self) G_GNUC_UNUSED;
static void _webi_finalize (GObject * self) G_GNUC_UNUSED;
static void _webi_realize (GtkWidget * widget) G_GNUC_UNUSED;
static void _webi_map (GtkWidget * widget) G_GNUC_UNUSED;
static void _webi_unmap (GtkWidget * widget) G_GNUC_UNUSED;
static gboolean _webi_expose_event (GtkWidget * widget, GdkEventExpose * event) G_GNUC_UNUSED;
static void _webi_size_allocate (GtkWidget * widget, GtkAllocation * allocation) G_GNUC_UNUSED;


enum {
    STATUS_TEXT_SIGNAL,
    STATUS_SIGNAL,
    LOCATION_SIGNAL,
    TITLE_SIGNAL,
    PROGRESS_SIGNAL,
    REQ_NEW_WINDOW,
    SHOW_WINDOW,
    CLOSE_WINDOW,
    SET_WINDOW_PROPERTIES,
    GET_WINDOW_SIZE,
    SET_WINDOW_SIZE,
    GET_DEVICE_TYPE,
    SET_DEVICE_TYPE,
    LOAD_START,
    LOAD_STOP,
    LOAD_ITEM_START,
    LOAD_ITEM_PROGRESS,
    LOAD_ITEM_STOP,
    SET_COOKIE,
    REQ_JS_PROMPT,
    REQ_AUTH_PROMPT,
    MOUSE_OVER,
    SELECTION,
    LAST_SIGNAL
};

static guint object_signals[LAST_SIGNAL] = {0};

/* pointer to the class of our parent */
static GtkBinClass *parent_class = NULL;

GType
webi_get_type (void)
{
    static GType type = 0;

    if G_UNLIKELY(type == 0) {
        static const GTypeInfo info = {
            sizeof (WebiClass),
            (GBaseInitFunc) NULL,
            (GBaseFinalizeFunc) NULL,
            (GClassInitFunc) webi_class_init,
            (GClassFinalizeFunc) NULL,
            NULL /* class_data */,
            sizeof (Webi),
            0 /* n_preallocs */,
            (GInstanceInitFunc) webi_init,
        };

        type = g_type_register_static (GTK_TYPE_BIN, "Webi", &info, (GTypeFlags)0);

		/* webi needs threading, because nrcit might use libsoup, which uses
		 * threads */
		if (!g_thread_supported ()) g_thread_init (NULL);
    }

    return type;
}

#undef PARENT_HANDLER
static void
webi_class_init (WebiClass * c)
{
    GObjectClass *g_object_class = (GObjectClass*) c;
    GtkWidgetClass *gtk_widget_class = (GtkWidgetClass *)c;

    parent_class = (GtkBinClass *)g_type_class_ref (GTK_TYPE_BIN);

    c->status = NULL;
    c->status_text = NULL;
    c->location = NULL;
    c->title = NULL;
    c->req_new_window = NULL;
    c->show_window = NULL;
    c->close_window = NULL;
    c->set_window_properties = NULL;
    c->get_window_size = NULL;
    c->set_window_size = NULL;
    c->get_device_type = NULL;
    c->set_device_type = NULL;
    c->load_start = NULL;
    c->load_stop = NULL;
    c->req_js_prompt = NULL;
    c->req_auth_prompt = NULL;

    object_signals[STATUS_TEXT_SIGNAL] =
        g_signal_new ("status_text",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, status_text),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
    object_signals[STATUS_SIGNAL] =
        g_signal_new ("status",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, status),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE, 1,
                      G_TYPE_POINTER);
    object_signals[LOCATION_SIGNAL] =
        g_signal_new ("location",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, location),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
    object_signals[TITLE_SIGNAL] =
        g_signal_new ("title",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, title),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
    object_signals[REQ_NEW_WINDOW] =
        g_signal_new ("req-new-window",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, req_new_window),
                      NULL, NULL,
                      webi_marshal_POINTER__POINTER,
                      G_TYPE_POINTER, 1, G_TYPE_POINTER);
    object_signals[SHOW_WINDOW] =
        g_signal_new ("show-window",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, show_window),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
    object_signals[CLOSE_WINDOW] =
        g_signal_new ("close-window",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, close_window),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
    object_signals[SET_WINDOW_PROPERTIES] =
        g_signal_new ("set-window-properties",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, set_window_properties),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE, 1, G_TYPE_POINTER);
    object_signals[GET_WINDOW_SIZE] =
        g_signal_new ("get-window-size",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, get_window_size),
                      NULL, NULL,
                      webi_marshal_VOID__POINTER_POINTER,
                      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
    object_signals[SET_WINDOW_SIZE] =
        g_signal_new ("set-window-size",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, set_window_size),
                      NULL, NULL,
                      webi_marshal_VOID__POINTER_POINTER,
                      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
    object_signals[GET_DEVICE_TYPE] =
        g_signal_new ("get-device-type",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, get_device_type),
                      NULL, NULL,
                      webi_marshal_INT__VOID,
                      G_TYPE_INT, 0);
    object_signals[SET_DEVICE_TYPE] =
        g_signal_new ("set-device-type",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, set_device_type),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__INT,
                      G_TYPE_NONE, 1, G_TYPE_INT);

    object_signals[LOAD_START] =
        g_signal_new ("load-start",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, load_start),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);

    object_signals[LOAD_STOP] =
        g_signal_new ("load-stop",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, load_stop),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);

    object_signals[SET_COOKIE] =
        g_signal_new ("set-cookie",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, set_cookie),
                      NULL, NULL,
                      webi_marshal_BOOLEAN__POINTER_POINTER,
                      G_TYPE_BOOLEAN, 2,
                      G_TYPE_POINTER, G_TYPE_POINTER);

    object_signals[REQ_JS_PROMPT] =
        g_signal_new ("req-js-prompt",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, req_js_prompt),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE, 1,
                      G_TYPE_POINTER);
    object_signals[REQ_AUTH_PROMPT] =
        g_signal_new ("req-auth-prompt",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, req_auth_prompt),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE, 1,
                      G_TYPE_POINTER);
    object_signals[MOUSE_OVER] =
        g_signal_new ("mouse-over",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, mouse_over),
                      NULL, NULL,
                      webi_marshal_VOID__POINTER_POINTER_POINTER_POINTER,
                      G_TYPE_NONE, 4,
                      G_TYPE_POINTER,G_TYPE_POINTER,G_TYPE_POINTER,G_TYPE_POINTER );
    object_signals[SELECTION] =
        g_signal_new ("selection",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebiClass, selection),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);

    g_object_class->finalize = _webi_finalize;
    gtk_widget_class->realize = _webi_realize;
    gtk_widget_class->map = _webi_map;
    gtk_widget_class->unmap = _webi_unmap;
    gtk_widget_class->expose_event = _webi_expose_event;
    gtk_widget_class->size_allocate = _webi_size_allocate;
    return;
}

static void
_webi_finalize(GObject *objself)
{
    Webi *self = WEBI (objself);
    gpointer priv = self->_priv;

    if (self->_priv->location) g_free (self->_priv->location);
    if (self->_priv->title) g_free (self->_priv->title);
    if (self->_priv->js_input_text) g_free (self->_priv->js_input_text);

    g_free (priv);

#define PARENT_HANDLER(___self) \
    { if(G_OBJECT_CLASS(parent_class)->finalize) \
        (* G_OBJECT_CLASS(parent_class)->finalize)(___self); }

    PARENT_HANDLER(G_OBJECT (self));
    return;
#undef PARENT_HANDLER
}

static void
_on_webi_destroy(GtkObject *object, gpointer user_data)

{
    Webi* self = WEBI (object);
    /* GtkBin has removed child from widget structure and decreased its reference count */
    /* destruct structures, which leads to destruction of child and all inner objects */

    delete(self->_priv->root);
    delete(self->_priv->sighandler);

}

static void
webi_init (Webi * self)
{
    self->_priv = g_new0 (WebiPrivate, 1);

    GTK_WIDGET_SET_FLAGS (self, GTK_CAN_FOCUS);

    OSB::Preferences *settings;
    self->_priv->root = new RootImpl(0,0);
    self->_priv->sighandler = new WebiRoot(self->_priv->root, self);
    self->_priv->root->setDefaultUIDelegate(deprecated_signal_sender_new(G_OBJECT (self)));

    self->_priv->child = self->_priv->root->mainFrame()->widget();
    settings = self->_priv->root->preferences();

    settings->setWillLoadImagesAutomatically(true);
    settings->setJavaScriptEnabled(true);
    settings->setJavaEnabled(false);
    settings->setPluginsEnabled(false);
    settings->setMinimumFontSize(6);
    settings->setDefaultFontSize(14);
    settings->setDefaultFixedFontSize(10);
    settings->setDefaultTextEncoding("ISO-8859-1");
    settings->setSerifFontFamily("serif");
    settings->setSansSerifFontFamily("sans");
    settings->setFixedFontFamily("fixed");
    settings->setStandardFontFamily("times");
    settings->setCursiveFontFamily("times");
    settings->setFantasyFontFamily("lucidabright");
    self->_priv->root->setDeviceType(OSB::DeviceScreen);

    GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET (self), GTK_NO_WINDOW);

    gtk_container_add (GTK_CONTAINER (self), self->_priv->child);

    /* to destruct structures (which destructs self->_priv->child)*/
    g_signal_connect (G_OBJECT (self),
                      "destroy",
                      G_CALLBACK (_on_webi_destroy),
                      self);

    return;
}

/**
 * webi_new:
 *
 * Makes a new #Webi widget
 *
 * Returns: a new widget
 **/
GtkWidget *
webi_new()
{
    Webi* self = static_cast<Webi*>(g_object_new (WEBI_TYPE_WEBI,  NULL));

    return (GtkWidget*)self;
}

void
webi_load_url (Webi * self, const gchar * url)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));

    self->_priv->root->mainFrame()->startLoad(url);
}

void
webi_refresh (Webi * self)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));

    self->_priv->root->mainFrame()->refresh();
}

void
webi_stop_load (Webi * self)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));

    self->_priv->root->mainFrame()->stopLoad();
}

gboolean
webi_can_go_back (Webi * self)
{
    g_return_val_if_fail (self != NULL, (gboolean )0);
    g_return_val_if_fail (WEBI_IS_WEBI (self), (gboolean )0);

    return self->_priv->root->canGoBack();
}

gboolean
webi_can_go_forward (Webi * self)
{
    g_return_val_if_fail (self != NULL, (gboolean )0);
    g_return_val_if_fail (WEBI_IS_WEBI (self), (gboolean )0);

    return self->_priv->root->canGoForward();
}

void
webi_go_back (Webi * self)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));

    if ( self->_priv->root->canGoBack())
        self->_priv->root->goBack();

}

void
webi_go_forward (Webi * self)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));

    if ( self->_priv->root->canGoForward())
        self->_priv->root->goForward();
}

const gchar *
webi_get_location (Webi * self)
{
    g_return_val_if_fail (self != NULL, (const gchar * )0);
    g_return_val_if_fail (WEBI_IS_WEBI (self), (const gchar * )0);

    return self->_priv->root->mainFrame()->location();
}

const gchar *
webi_get_title (Webi * self)
{
    g_return_val_if_fail (self != NULL, (const gchar * )0);
    g_return_val_if_fail (WEBI_IS_WEBI (self), (const gchar * )0);


    return self->_priv->sighandler->myTitle();
}


const WebiLoadStatus *
webi_get_status (Webi * self)
{
    g_return_val_if_fail (self != NULL, (const WebiLoadStatus * )0);
    g_return_val_if_fail (WEBI_IS_WEBI (self), (const WebiLoadStatus * )0);

    return self->_priv->sighandler->myStatus();
}

void*
webi_get_internal (Webi * self)
{
    g_return_val_if_fail (self != NULL, NULL);
    g_return_val_if_fail (WEBI_IS_WEBI (self), NULL);


    return self->_priv->root;
}


void webi_set_settings (Webi * self, const WebiSettings * settings)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));
    OSB::Preferences *s = self->_priv->root->preferences();

    s->setWillLoadImagesAutomatically (settings->autoload_images);
    s->setJavaScriptEnabled (settings->javascript_enabled);
    s->setJavaEnabled (settings->java_enabled);
    s->setPluginsEnabled (settings->plugins_enabled);
    s->setMinimumFontSize (settings->minimum_font_size);
    s->setDefaultFontSize (settings->default_font_size);
    s->setDefaultFixedFontSize (settings->default_fixed_font_size);

#define CALL_IF_SET(inst, method, var) if ((var)) (inst)->method (var);

    CALL_IF_SET(s, setDefaultTextEncoding, settings->default_text_encoding);
    CALL_IF_SET(s, setSerifFontFamily, settings->serif_font_family);
    CALL_IF_SET(s, setSansSerifFontFamily, settings->sans_serif_font_family);
    CALL_IF_SET(s, setFixedFontFamily, settings->fixed_font_family);
    CALL_IF_SET(s, setStandardFontFamily, settings->standard_font_family);
    CALL_IF_SET(s, setCursiveFontFamily, settings->cursive_font_family);
    CALL_IF_SET(s, setFantasyFontFamily, settings->fantasy_font_family);
#undef CALL_IF_SET

    OSB::ProtocolDelegate* httpprov = self->_priv->root->protocolDelegateForURL("http://");
    if (httpprov) {
        httpprov->setProxy(settings->http_proxy);
    }

    OSB::DeviceType type = OSB::DeviceScreen;
    switch (settings->device_type) {
    case WEBI_DEVICE_TYPE_HANDHELD:
        type = OSB::DeviceHandheld;
        break;
    case WEBI_DEVICE_TYPE_PRINTER:
        type = OSB::DevicePrinter;
        break;
    case WEBI_DEVICE_TYPE_SCREEN:
        /* fallthrough */
    default:
        break;
    }
    self->_priv->root->setDeviceType(type);

    if (settings->user_agent_string) {
        self->_priv->root->setUserAgentString(settings->user_agent_string);
    }
}

void
webi_set_emit_internal_status (Webi* self, gboolean flag)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));

    self->_priv->sighandler->setEmitInternal(flag == TRUE);
}

void
webi_set_device_type (Webi * self, WebiDeviceType device)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));

    OSB::DeviceType type;
    switch (device) {
    case WEBI_DEVICE_TYPE_HANDHELD:
        type = OSB::DeviceHandheld;
        break;
    case WEBI_DEVICE_TYPE_SCREEN:
        /* fallthrough */
    case WEBI_DEVICE_TYPE_PRINTER:
        /* fallthrough */
    default:
        type = OSB::DeviceScreen;
        break;
    }

    self->_priv->root->setDeviceType(type);
}

/** Gets the device type used for current window.
 *
 * @param self the engine which sent the signal
 * @param newDeviceType The new device type of current window
 */
WebiDeviceType
webi_get_device_type (Webi * self) {
    RootImpl* root = self->_priv->root;
    OSB::DeviceType device = root->deviceType();
    switch (device)
    {
    case OSB::DeviceHandheld:
        return WEBI_DEVICE_TYPE_HANDHELD;
        break;
    case OSB::DevicePrinter:
        return WEBI_DEVICE_TYPE_PRINTER;
        break;
    default:
        /** fallthrough*/
    case OSB::DeviceScreen:
        break;
    }

    return WEBI_DEVICE_TYPE_SCREEN;
}


void
webi_set_group (Webi * self, const gchar* group)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));

    RootImpl* root = self->_priv->root;
    root->setGroup(group);
}

const gchar *
webi_get_group (Webi * self)
{
    g_return_val_if_fail (self != NULL, NULL);
    g_return_val_if_fail (WEBI_IS_WEBI (self), NULL);

    RootImpl* root = self->_priv->root;
    return root->group();
}

void
webi_set_text_multiplier (Webi* self, gfloat multiplier)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));

    RootImpl* root = self->_priv->root;
    root->setTextSizeMultiplier(multiplier);
}

gfloat
webi_get_text_multiplier (Webi* self)
{
    g_return_val_if_fail (self != NULL, -1.0);
    g_return_val_if_fail (WEBI_IS_WEBI (self), -1.0);

    RootImpl* root = self->_priv->root;
    return root->textSizeMultiplier();
}

gboolean
webi_find (Webi* self, const gchar* text, gboolean case_sensitive, gboolean dir_down)
{
    g_return_val_if_fail (self != NULL, FALSE);
    g_return_val_if_fail (WEBI_IS_WEBI (self), FALSE);

    return self->_priv->root->searchFor(text, dir_down, case_sensitive);
}

gchar*
webi_get_current_selection_as_text(Webi* self)
{
    g_return_val_if_fail (self != NULL, NULL);
    g_return_val_if_fail (WEBI_IS_WEBI (self), NULL);

    return self->_priv->root->selectedText();
}

static void
_webi_realize (GtkWidget * widget)
#define PARENT_HANDLER(___widget) \
    { if(GTK_WIDGET_CLASS(parent_class)->realize) \
        (* GTK_WIDGET_CLASS(parent_class)->realize)(___widget); }
{


    GdkWindowAttr   attributes;
    gint            attributes_mask;

    g_return_if_fail(widget != NULL);

    GTK_WIDGET_SET_FLAGS(widget, GTK_REALIZED);

    attributes.window_type = GDK_WINDOW_CHILD;
    attributes.x = widget->allocation.x;
    attributes.y = widget->allocation.y;
    attributes.width = widget->allocation.width;
    attributes.height = widget->allocation.height;
    attributes.wclass = GDK_INPUT_OUTPUT;
    attributes.visual = gtk_widget_get_visual (widget);
    attributes.colormap = gtk_widget_get_colormap (widget);
    attributes.event_mask = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK;

    attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

    widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
                                     &attributes, attributes_mask);
    gdk_window_set_user_data (widget->window, widget);

    widget->style = gtk_style_attach(widget->style, widget->window);

}
#undef PARENT_HANDLER

static void
_webi_map (GtkWidget * widget)
#define PARENT_HANDLER(___widget) \
    { if(GTK_WIDGET_CLASS(parent_class)->map) \
        (* GTK_WIDGET_CLASS(parent_class)->map)(___widget); }
{

    GtkWidget* child;
    g_return_if_fail(widget != NULL);
    g_return_if_fail(WEBI_IS_WEBI (widget));

    child = GTK_BIN(widget)->child;

    GTK_WIDGET_SET_FLAGS(widget, GTK_MAPPED);

    //NEEDED?
    if (GTK_WIDGET_VISIBLE(child))
    {
        if (!GTK_WIDGET_MAPPED (child)){
            gtk_widget_map(child);
        }
    }
    gdk_window_show(widget->window);
}
#undef PARENT_HANDLER

static void
_webi_unmap (GtkWidget * widget)
{

    g_return_if_fail(widget != NULL);
    g_return_if_fail(WEBI_IS_WEBI (widget));

    GTK_WIDGET_UNSET_FLAGS(widget, GTK_MAPPED);

    gdk_window_hide(widget->window);
}

static gboolean
_webi_expose_event (GtkWidget * widget, GdkEventExpose * event)
#define PARENT_HANDLER(___widget,___event) \
    ((GTK_WIDGET_CLASS(parent_class)->expose_event)? \
        (* GTK_WIDGET_CLASS(parent_class)->expose_event)(___widget,___event): \
        ((gboolean )0))
{

    return PARENT_HANDLER(widget,event);
}

#undef PARENT_HANDLER

static void
_webi_size_allocate (GtkWidget * widget, GtkAllocation * allocation)
#define PARENT_HANDLER(___widget,___allocation) \
    { if(GTK_WIDGET_CLASS(parent_class)->size_allocate) \
        (* GTK_WIDGET_CLASS(parent_class)->size_allocate)(___widget,___allocation); }
{

    GtkWidget* child;
    g_return_if_fail(widget != NULL);
    g_return_if_fail(WEBI_IS_WEBI (widget));

    widget->allocation = *allocation;

    // NEEDED? Not in Mozilla
    child = GTK_BIN(widget)->child;
    if (child){
        GtkAllocation a;
        GtkRequisition r;
        gtk_widget_size_request(child, &r);

        a.x = 0;
        a.y = 0;
        /* a.height = r.height; */
        /*  a.width = r.width; */

        a.width = allocation->width;
        a.height = allocation->height;

        gtk_widget_size_allocate(child, &a);
    }

    if (GTK_WIDGET_REALIZED(widget))
    {
        gdk_window_move_resize(widget->window,
                               allocation->x, allocation->y,
                               allocation->width, allocation->height);
    }
}
#undef PARENT_HANDLER



const gchar *
webi_render_tree (Webi * self)
{
    g_return_val_if_fail (self != NULL, NULL);
    g_return_val_if_fail (WEBI_IS_WEBI (self), NULL);

    //    self->_priv->root->aux().renderTreeString();
    return "";
}

const gchar *
webi_get_engine_user_agent_string ()
{
    return BridgeImpl::engineUserAgentString();
}


void
webi_set_default_external_content_delegate (Webi* self, WebiExternalContentDelegate* delegate)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBI_IS_WEBI (self));

    RootImpl* root = self->_priv->root;
    root->setDefaultExternalContentDelegate(delegate);
}

WebiExternalContentDelegate* webi_get_default_external_content_delegate (Webi* self)
{
    g_return_val_if_fail (self != NULL, NULL);
    g_return_val_if_fail (WEBI_IS_WEBI (self), NULL);

    RootImpl* root = self->_priv->root;
    return root->getDefaultExternalContentDelegate();
}
