/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* Copyright (C) 2007 Kimmo Kinnunen <kimmok@iki.fi>
 * bsd license
 */

#include "MIMETypeResolver.h"

using namespace Loader;

MIMETypeResolver::MIMETypeResolver()
{
}

MIMETypeResolver::~MIMETypeResolver()
{
}

/** mime-type resolver for files.
 * one can conditionally compile in better resolvers
 * by implementing one with for example gnome-vfs.
 * if this is done, the implementation below will need to
 * be moved to its own file.
 */
class SimpleMIMETypeResolver: public MIMETypeResolver {
public:
	SimpleMIMETypeResolver() {}
	~SimpleMIMETypeResolver() {}
	const gchar* mimeTypeForURL(const gchar* path) const;
};

MIMETypeResolver& MIMETypeResolver::sharedFactory()
{
	static SimpleMIMETypeResolver singleton;
	return singleton;
}

const gchar* SimpleMIMETypeResolver::mimeTypeForURL(const gchar* url) const
{
	const gchar* retval = "application/octet-stream";

	gchar* lurl = g_utf8_strdown(url, -1);

    if (g_str_has_prefix(lurl, "file:")) {
        if (g_str_has_suffix(lurl, ".html")
            || g_str_has_suffix(lurl, ".htm")) {
            retval = "text/html";
        } else if (g_str_has_suffix(lurl, ".xhtml")
                   || g_str_has_suffix(lurl, ".xml")
                   || g_str_has_suffix(lurl, ".rss")) {
            retval = "text/xml";
        } else if (g_str_has_suffix(lurl, ".css")) {
            retval = "text/css";
        } else if (g_str_has_suffix(lurl, ".js")) {
            retval = "text/javascript";
        } else if (g_str_has_suffix(lurl, ".png")) {
            retval = "image/png";
        } else if (g_str_has_suffix(lurl, ".gif")) {
            retval = "image/gif";
        } else if (g_str_has_suffix(lurl, ".jpg")
                   || g_str_has_suffix(lurl, ".jpeg")) {
            retval = "image/jpeg";
        }
    }

	g_free(lurl);
	return retval;
}

