/*
 * Copyright (c) 2004 Nokia. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Nokia nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef rootimpl_h
#define rootimpl_h

#include <glib.h>

#include "osb.h"
#include "webkit-gtk/webkit-view.h"
#include "gtk/webi-external-content-delegate.h"
#include "webkit-gtk/webkit-ui-delegate.h"
#include "ref_gobjectptr.h"

class BridgeImpl;
class FrameImpl;
class PreferencesImpl;

class FrameGroup;
class WebCoreSettings;


class RootImpl
{
public:
    RootImpl(const gchar* frameName, const gchar* group);
	RootImpl(WebkitView*); 		// new style 
    virtual ~RootImpl();

    void setGroup(const gchar*);
    const gchar* group();
    
    FrameImpl* mainFrame();
    
    /** @return GList of BridgeImpl*
     */
    GList* peers();

    OSB::BackForwardList* backForwardList();

    void goBack();
    bool canGoBack();
    void goForward();
    bool canGoForward();

    /** Sets the device type to be used. */
    void setDeviceType(OSB::DeviceType deviceType);

    /** Gets the device type currently used in rendering. */
    OSB::DeviceType deviceType() const;

    void setTextSizeMultiplier(float multiplier);

    float textSizeMultiplier();
    
    void setResourceLoadDelegate(OSB::ResourceLoadDelegate*);

    OSB::ProtocolDelegate* protocolDelegateForURL(const gchar* url);
    
    bool searchFor(const gchar *string, bool forward, bool caseSensitive);

    void setPreferences(const OSB::Preferences& prefs);
    OSB::Preferences*  preferences();

    void setFeatures(const OSB::Features& prefs);
    OSB::Features*  features();
    
    WebCoreSettings* sharedSettings();
    WebCoreSettings* sharedSettings() const;

    void setUserAgentString(const gchar*);
    const gchar* userAgentForURL(const gchar* URL);
    gchar* selectedText();

	void setFrameLoadDelegate(OSB::FrameLoadDelegate* d);
	void setOldUIDelegate(OSB::UIDelegate* d);

	void setDefaultExternalContentDelegate(WebiExternalContentDelegate* delegate) { m_externalContentDelegate = delegate; }
	WebiExternalContentDelegate* getDefaultExternalContentDelegate() {return m_externalContentDelegate;}

    void setDefaultUIDelegate(WebkitUIDelegate* delegate);
    WebkitUIDelegate* getDefaultUIDelegate();

	WebkitView* api() { return m_api; }
	
private:
    FrameImpl* _mainFrame;

    float _textSizeMultiplier;

    FrameGroup* _currentGroup;
    OSB::Preferences prefs;
    OSB::Features feats;
    OSB::DeviceType _deviceType;
	WebkitView* m_api;

	ref_gobjectptr<WebiExternalContentDelegate> m_externalContentDelegate;
};

#endif
