/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* Copyright (C) 2007 Kimmo Kinnunen<kimmok@iki.fi>
 * bsd license
 */
#include "nrcittest/NRCitTest.h"
#include <assert.h>
#include <string.h>
#include "CookieParser.h"
#include "Cookie.h"


int RunUnitTests()
{
    nrcittest::TestSet ts;
    CookieParser p;
    const gchar* set1 = "Customer=\"WILE_E_COYOTE\"; Version=\"1\"; Path=\"/acme\"";

	Cookie** av = p.parse(set1);
	Cookie* c = av[0];

#define TSTSTREQ(prop, str) ts.test(c->prop() && strcmp(c->prop(), (str))==0, str, c->prop())

	TSTSTREQ(name, "Customer");
	TSTSTREQ(value, "WILE_E_COYOTE");
	TSTSTREQ(path, "/acme");
	ts.test(c->version() == 1, 1, c->version());
	assert(c);

    const gchar* set2 = "Customer=\"WILE_E_COYOTE\"; Version=\"1\"; Path=\"/acme\"; Domain=\".example.com\";Secure;Discard;Max-Age=\"0\";Comment=\"Comment\";CommentURL=\"CommentURL\"";

	av = p.parse(set2);
	c = av[0];
	assert(c);

	TSTSTREQ(name, "Customer");
	TSTSTREQ(value, "WILE_E_COYOTE");
	TSTSTREQ(path, "/acme");
	TSTSTREQ(domain, ".example.com");
	TSTSTREQ(comment, "Comment");
	TSTSTREQ(commentURL, "CommentURL");
	ts.test(c->version() == 1, 1, c->version());
	ts.test(c->secure() == true, 1, c->secure());
	ts.test(c->discard() == true, 1, c->discard());

	const gchar* set3 = "Customer=W34E_5_C56OTE; Version=1; Path=\"/acme\"; Domain=.example.com;Secure;Discard;Max-Age=0;Comment=Comment;CommentURL=CommentURL";
	av = p.parse(set3);
	c = av[0];
	assert(c);

	TSTSTREQ(name, "Customer");
	TSTSTREQ(value, "W34E_5_C56OTE");
	TSTSTREQ(path, "/acme");
	TSTSTREQ(domain, ".example.com");
	TSTSTREQ(comment, "Comment");
	TSTSTREQ(commentURL, "CommentURL");
	ts.test(c->version() == 1, 1, c->version());
	ts.test(c->secure() == true, 1, c->secure());
	ts.test(c->discard() == true, 1, c->discard());

	const gchar* set4 = "A1_B4=W-zz; Version=1; Path=\"/acme\"; Domain=.example.com;Secure;Discard;Max-Age=0;Comment=Comment;CommentURL=CommentURL";
	av = p.parse(set4);
	c = av[0];
	assert(c);

	TSTSTREQ(name, "A1_B4");
	TSTSTREQ(value, "W-zz");
	TSTSTREQ(path, "/acme");
	TSTSTREQ(domain, ".example.com");
	TSTSTREQ(comment, "Comment");
	TSTSTREQ(commentURL, "CommentURL");
	ts.test(c->version() == 1, 1, c->version());
	ts.test(c->secure() == true, 1, c->secure());
	ts.test(c->discard() == true, 1, c->discard());

	const gchar* set5 = "B=W;Path=\"/\"; Domain=example.com;Max-Age=110;unkn=\"22\";Port=\"80\";unkn=13";
	av = p.parse(set5);
	c = av[0];
	assert(c);

	TSTSTREQ(name, "B");
	TSTSTREQ(value, "W");
	TSTSTREQ(path, "/");
	TSTSTREQ(domain, "example.com");
	ts.test(c->maxAge() == 110, 1, c->version());
	const int* ports = c->ports();
	ts.test( ports && c->nports() == 1 && *ports == 80, 80, ports ? -1:*ports);

	const gchar* set6 = "A=1,B=W;Path=\"/\" ,zz=top";
	av = p.parse(set6);
	assert(av[0]);
	assert(av[1]);
	assert(av[2]);
    c = av[0];
	TSTSTREQ(name, "A");
	TSTSTREQ(value, "1");
    c = av[1];
	TSTSTREQ(name, "B");
	TSTSTREQ(value, "W");
	TSTSTREQ(path, "/");
    c = av[2];
	TSTSTREQ(name, "zz");
	TSTSTREQ(value, "top");



    return ts.failures();
}

