/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* Copyright (C) 2007 Kimmo Kinnunen<kimmok@iki.fi>
 * bsd license
 */

#include <string.h>
#include "CookieStorage.h"
#include "nrcittest/NRCitTest.h"

int RunUnitTests()
{
    nrcittest::TestSet ts;
    CookieStorage storage;
    const gchar* set1 = "Customer=\"WILE_E_COYOTE\"; Version=\"1\"; Path=\"/acme\"";
    const gchar* exp1 = "Cookie: $Version=\"1\";Customer=\"WILE_E_COYOTE\";$Path=\"/acme\"";
    const gchar* domain1 = "http://www.example.com/";
    const gchar* domain2 = "http://www.example.com/acme/login";
    storage.setCookiesForURL(set1, domain1, 0);
    const gchar* res;
	res = storage.cookiesForURL(domain1);
    ts.test(!res, "null", res);

    storage.setCookiesForURL(set1, domain2, 0);

    res = storage.cookiesForURL(domain2);
    ts.test(res && strcmp(exp1, res)==0, exp1, res);

    res = storage.cookiesForURL(domain1);
    ts.test(!res, "null", res);

	const gchar* domain3 = "http://www.example.com/acme/pickitem";
	res = storage.cookiesForURL(domain3);
	ts.test(res && strcmp(exp1, res)==0, exp1, res);

	const gchar* set2 = "Part_Number=\"Rocket_Launcher_0001\"; Version=\"1\"; Path=\"/acme\"";
	storage.setCookiesForURL(set2, domain3, 0);

    const gchar* exp2 = "Cookie: $Version=\"1\";Customer=\"WILE_E_COYOTE\";$Path=\"/acme\";Part_Number=\"Rocket_Launcher_0001\";$Path=\"/acme\"";
	const gchar* domain4 = "http://www.example.com/acme/shipping";
	res = storage.cookiesForURL(domain4);
	ts.test(res && strcmp(exp2, res)==0, exp2, res);

    const gchar* exp3 = "Cookie: ad=buy";
	const gchar* set3 = "ad=buy;";
	storage.setCookiesForURL(set3, domain1, 0);
	res = storage.cookiesForURL(domain1);
	ts.test(res && strcmp(exp3, res)==0, exp3, res);

	const gchar* set4 = "Part_Number=\"Rocket_Launcher_0004\"; Version=\"1\"; Path=\"/acme\"";
    const gchar* exp4 = "Cookie: $Version=\"1\";Customer=\"WILE_E_COYOTE\";$Path=\"/acme\";Part_Number=\"Rocket_Launcher_0004\";$Path=\"/acme\""
		"\r\nCookie: ad=buy";
	storage.setCookiesForURL(set4, domain3, 0);
	res = storage.cookiesForURL(domain4);
	ts.test(res && strcmp(exp4, res)==0, exp4, res);

    return ts.failures();
}

