/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (c) 2004 Nokia. All rights reserved.
 * Copyright (c) 2007 Kimmo Kinnunen. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Nokia nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "webkit-frame.h"
#include "webkit-frame-private.h"
#include "webkit-url-request.h"

#include "frameimpl.h"
#include "rootimpl.h"

G_DEFINE_TYPE(WebkitFrame, webkit_frame, G_TYPE_OBJECT);

#define WEBKIT_FRAME_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), WEBKIT_TYPE_FRAME, WebkitFramePrivate))
#define SELF_PRIV(self) WebkitFramePrivate* priv = WEBKIT_FRAME_GET_PRIVATE(self)

static void webkit_frame_finalize(WebkitFrame *self);

static void webkit_frame_init (WebkitFrame *self)
{

}

static void webkit_frame_class_init (WebkitFrameClass *klass)
{
	/* register private structure */
	g_type_class_add_private(klass, sizeof(WebkitFramePrivate));
	/* bind finalizing function */
	G_OBJECT_CLASS (klass)->finalize = (GObjectFinalizeFunc) webkit_frame_finalize;
}


static void webkit_frame_finalize(WebkitFrame *self)
{
}

WebkitFrame* webkit_frame_new(FrameImpl* owner)
{
	WebkitFrame* self = WEBKIT_FRAME(g_object_new (WEBKIT_TYPE_FRAME,  NULL));
	SELF_PRIV(self);
	priv->frame = owner;

	return self;
}

const gchar* webkit_frame_get_name(WebkitFrame* self)
{
	g_return_val_if_fail(WEBKIT_IS_FRAME(self), NULL);
	SELF_PRIV(self);

	return priv->frame->getFrameName();
}

WebkitView* webkit_frame_get_view(WebkitFrame* self)
{
	g_return_val_if_fail(WEBKIT_IS_FRAME(self), NULL);
	SELF_PRIV(self);

	return priv->frame->root()->api();
}

WebkitFrameView* webkit_frame_get_frame_view(WebkitFrame* self)
{
	g_return_val_if_fail(WEBKIT_IS_FRAME(self), NULL);
	//SELF_PRIV(self);
	assert(0);
	// not implemented

	return 0;
}

void webkit_frame_load_request(WebkitFrame* self, WebkitURLRequest* request)
{
	webkit_frame_load_http_get_str(self, webkit_url_request_get_url_str(request));
}

void webkit_frame_load_http_get_str(WebkitFrame* self, const gchar* url)
{
    g_return_if_fail (WEBKIT_IS_FRAME (self));
	g_return_if_fail (url);
	SELF_PRIV(self);

    priv->frame->root()->mainFrame()->startLoad(url);
}

void webkit_frame_load_html_string(WebkitFrame* self, const gchar* string, const gchar* base_url)
{
    g_return_if_fail (WEBKIT_IS_FRAME (self));
	g_return_if_fail (string);
	g_return_if_fail (base_url);
	assert(0);
	// not implemented
}


void webkit_frame_stop_loading(WebkitFrame* self)
{
    g_return_if_fail (WEBKIT_IS_FRAME (self));

	SELF_PRIV(self);
    priv->frame->root()->mainFrame()->stopLoad();
}

void webkit_frame_reload(WebkitFrame* self)
{
    g_return_if_fail (WEBKIT_IS_FRAME (self));

	SELF_PRIV(self);
    priv->frame->root()->mainFrame()->refresh();
}

void webkit_frame_set_ui_delegate(WebkitFrame* self, WebkitUIDelegate* delegate)
{
	g_return_if_fail (WEBKIT_IS_FRAME (self));
	SELF_PRIV (self);
	priv->frame->setUIDelegate(delegate);
}

WebkitUIDelegate* webkit_get_ui_delegate(WebkitFrame* self)
{
	g_return_val_if_fail (WEBKIT_IS_FRAME (self), NULL);
	SELF_PRIV (self);
	return priv->frame->getUIDelegate();
}

void webkit_set_resource_load_delegate(WebkitFrame* self, WebkitResourceLoadDelegate* delegate)
{
    g_return_if_fail (WEBKIT_IS_FRAME (self));
	//SELF_PRIV(self);
	assert(0);
}

WebkitResourceLoadDelegate* webkit_get_resource_load_delegate(WebkitFrame* self)
{
    g_return_val_if_fail (WEBKIT_IS_FRAME (self), NULL);
	//SELF_PRIV(self);
	assert(0);
	return NULL;
}

void webkit_set_frame_load_delegate(WebkitFrame* self, WebkitFrameLoadDelegate* delegate)
{
    g_return_if_fail (WEBKIT_IS_FRAME (self));
	//SELF_PRIV(self);
	assert(0);
}

WebkitFrameLoadDelegate* webkit_get_frame_load_delegate(WebkitFrame* self)
{
    g_return_val_if_fail (WEBKIT_IS_FRAME (self), NULL);
	//SELF_PRIV(self);
	assert(0);

	return NULL;
}




