/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/* Copyright (c) 2007 Kimmo Kinnunen. All rights reserved.
 * bsd license
 */

#ifndef webkit_ui_delegate_h
#define webkit_ui_delegate_h

#include <glib.h>
#include <glib-object.h>

G_BEGIN_DECLS

#include <webkit-gtk/webkit-types.h>

#define WEBKIT_TYPE_UI_DELEGATE (webkit_ui_delegate_get_type())
#define WEBKIT_UI_DELEGATE(obj)   G_TYPE_CHECK_INSTANCE_CAST((obj), webkit_ui_delegate_get_type(), WebkitUIDelegate)
#define WEBKIT_UI_DELEGATE_CONST(obj) G_TYPE_CHECK_INSTANCE_CAST((obj), webkit_ui_delegate_get_type(), WebkitUIDelegate const)
#define WEBKIT_IS_UI_DELEGATE(obj)   G_TYPE_CHECK_INSTANCE_TYPE((obj), webkit_ui_delegate_get_type ())
#define WEBKIT_UI_DELEGATE_GET_IFACE(obj) G_TYPE_INSTANCE_GET_INTERFACE((obj), webkit_ui_delegate_get_type(), WebkitUIDelegateInterface)

struct WebkitUIDelegateInterface_ {
	GTypeInterface parent;

	WebkitView* (*create_view_with_request)(WebkitUIDelegate* self, WebkitView* sender, WebkitURLRequest* request);

	void (*show_view)(WebkitUIDelegate* self, WebkitView* sender);

	void (*close_view)(WebkitUIDelegate* self, WebkitView* sender);

	void (*set_status_text)(WebkitUIDelegate* self, WebkitView* sender, const gchar* status_text);

	const gchar* (*get_status_text)(WebkitUIDelegate* self, WebkitView* sender);

	void (*run_javascript_alert_panel_with_message)(WebkitUIDelegate* self, WebkitView* sender, const gchar* message, WebkitFrame *initiator_frame);

	gboolean (*run_javascript_confirm_panel_with_message)(WebkitUIDelegate* self, WebkitView* sender, const gchar* message, WebkitFrame *initiator_frame);

	gchar* (*run_javascript_input_panel_with_prompt)(WebkitUIDelegate* self, WebkitView* sender, const gchar* message, const gchar* default_text, WebkitFrame *initiator_frame);

	gboolean (*mouse_action_on_element)(WebkitUIDelegate* self, WebkitView* sender, WebkitElementInfo* element_info, WebkitMouseEventInfo* mouse_info);
};

GType webkit_ui_delegate_get_type();

WebkitView* webkit_ui_delegate_create_view_with_request(WebkitUIDelegate* self, WebkitView* sender, WebkitURLRequest* request);

void webkit_ui_delegate_show_view(WebkitUIDelegate* self, WebkitView* sender);

void webkit_ui_delegate_close_view(WebkitUIDelegate* self, WebkitView* sender);

void webkit_ui_delegate_set_status_text(WebkitUIDelegate* self, WebkitView* sender, const gchar* status_text);

const gchar* webkit_ui_delegate_get_status_text(WebkitUIDelegate* self, WebkitView* sender);

void webkit_ui_delegate_run_javascript_alert_panel_with_message(WebkitUIDelegate* self, WebkitView* sender, const gchar* message, WebkitFrame *initiator_frame);

gboolean webkit_ui_delegate_run_javascript_confirm_panel_with_message(WebkitUIDelegate* self, WebkitView* sender, const gchar* message, WebkitFrame *initiator_frame);

/**
 * @return The result string if ok was pressed, NULL otherwise.
 *         The ownership of result string is transferred and it will be freed with
 *         g_free ()
 */
gchar* webkit_ui_delegate_run_javascript_input_panel_with_prompt(WebkitUIDelegate* self, WebkitView* sender, const gchar* message, const gchar* default_text, WebkitFrame *initiator_frame);

/**
 * @return \TRUE if event was consumed \FALSE if event should be handled by the engine
 * @note: not part of Apple WebView API
 */
gboolean webkit_ui_delegate_mouse_action_on_element(WebkitUIDelegate* self, WebkitView* sender, WebkitElementInfo* element_info, WebkitMouseEventInfo* mouse_info);

G_END_DECLS

#endif
